/**************************************************\
* Copyright 2011:                                  *
* William McGuire                                  *
* pantsless@waronpants.net                         *
* http://waronpants.net                            *
*                                                  *
* Originally posted at:                            *
* http://waronpants.net/p/419                      *
*                                                  *
* Use at your own risk                             *
\**************************************************/

/**************************************************\
*                   Change Log                     *
****************************************************
*                                                  *
* April 28, 2011 - v1.1                            *
* Multiple files can be processed in one execution *
* Fixed newline conversion error                   *
*                                                  *
* March 18, 2011 - v1.0                            *
* Original version                                 *
*                                                  *
\**************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main (int argc, char * argv [])
{
    if (argc < 2)
    {
        printf("Error: Must include at least one input file.\n");
        return -1;
    }
    
    int file_index;
    for (file_index = 1; file_index < argc; ++file_index)
    {
        FILE * infile = fopen(argv[file_index], "rb");
        
        if (! infile)
        {
            printf("Can't open input file: %s\n", argv[file_index]);
            return -1;
        }
        
        char * outfilename = (char *)malloc(strlen(argv[file_index]) + 9);
        sprintf(outfilename, "%s-bin.txt", argv[file_index]);
        FILE * outfile = fopen(outfilename, "wb");
        
        if (! outfile)
        {
            printf("Can't open output file: %s\n", outfilename);
            free(outfilename);
            fclose(infile);
            return -1;
        }
        
        int hexdigit;
        int even_digit = 1;
        char outchar = 0;
        while ( (hexdigit = fgetc(infile)) != EOF )
        {
            char nybble;
            switch (hexdigit)
            {
            case '0':
                nybble = 0x00;
                break;
            case '1':
                nybble = 0x01;
                break;
            case '2':
                nybble = 0x02;
                break;
            case '3':
                nybble = 0x03;
                break;
            case '4':
                nybble = 0x04;
                break;
            case '5':
                nybble = 0x05;
                break;
            case '6':
                nybble = 0x06;
                break;
            case '7':
                nybble = 0x07;
                break;
            case '8':
                nybble = 0x08;
                break;
            case '9':
                nybble = 0x09;
                break;
            case 'A':
            case 'a':
                nybble = 0x0A;
                break;
            case 'B':
            case 'b':
                nybble = 0x0B;
                break;
            case 'C':
            case 'c':
                nybble = 0x0C;
                break;
            case 'D':
            case 'd':
                nybble = 0x0D;
                break;
            case 'E':
            case 'e':
                nybble = 0x0E;
                break;
            case 'F':
            case 'f':
                nybble = 0x0F;
                break;
            default:
                continue;
            }
            
            even_digit = (even_digit?0:1);
            
            outchar &= (even_digit?0xF0:0x0F);
            outchar |= (even_digit?nybble:(nybble<<4));
            
            if (even_digit)
            {
                char binstring [9];
                binstring[8] = 0;
                int i;
                for (i = 7; i >= 0; --i)
                {
                    binstring[7-i] = ( (outchar & (1<<i)) ? '1' : '0' );
                }
                fprintf(outfile, "%s ", binstring);
            }
        }
        
        free(outfilename);
        fclose(infile);
        fclose(outfile);
    }
}
