#!/usr/bin/perl

use warnings;
use strict;

##### Start User Deifined Variables #####
my $char2bin_executable = '../char2bin.exe';
my $char2hex_executable = '../char2hex.exe';
my $bin2char_executable = '../bin2char.exe';
my $bin2hex_executable = '../bin2hex.exe';
my $hex2char_executable = '../hex2char.exe';
my $hex2bin_executable = '../hex2bin.exe';

my @original_char_files = ('fox.txt', 'lorem_ipsum.txt');
my @original_bin_files = ('fox.bin', 'lorem_ipsum.bin');
my @original_hex_files = ('fox.hex', 'lorem_ipsum.hex');

my $output_char_file_suffix = '-char.txt';
my $output_bin_file_suffix = '-bin.txt';
my $output_hex_file_suffix = '-hex.txt';
#####  End User Deifined Variables  #####


# Generate output filenames
my @output_char2bin_files;
my @output_char2hex_files;
my @output_bin2char_files;
my @output_bin2hex_files;
my @output_hex2char_files;
my @output_hex2bin_files;

foreach (@original_char_files)
{
    push(@output_char2bin_files, $_ . $output_bin_file_suffix);
    push(@output_char2hex_files, $_ . $output_hex_file_suffix);
}

foreach (@original_bin_files)
{
    push(@output_bin2char_files, $_ . $output_char_file_suffix);
    push(@output_bin2hex_files, $_ . $output_hex_file_suffix);
}

foreach (@original_hex_files)
{
    push(@output_hex2char_files, $_ . $output_char_file_suffix);
    push(@output_hex2bin_files, $_ . $output_bin_file_suffix);
}

# Display expected file names
print "Expected output file names:\n";

foreach (@output_char2bin_files, @output_char2hex_files, @output_bin2char_files, @output_bin2hex_files, @output_hex2char_files, @output_hex2bin_files)
{
    print "$_\n";
}

print "\n";

# Turn on slurp mode
$/ = '';
$\ = '';


# char2bin tests
for (my $index = 0; $index <= $#original_char_files; ++$index)
{
    my $command = "$char2bin_executable $original_char_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_bin_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_char2bin_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_bin_files[$index] to $output_char2bin_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_char2bin_files[$index];
    
    print "\n";
}

# char2bin multiple-file test
{
    my $command = "$char2bin_executable @original_char_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_char_files; ++$index)
    {
        open(ORIGINAL, '<', $original_bin_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_char2bin_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_bin_files[$index] to $output_char2bin_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_char2bin_files;
    
    print "\n";
}


# char2hex tests
for (my $index = 0; $index <= $#original_char_files; ++$index)
{
    my $command = "$char2hex_executable $original_char_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_hex_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_char2hex_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_hex_files[$index] to $output_char2hex_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_char2hex_files[$index];
    
    print "\n";
}

# char2hex multiple-file test
{
    my $command = "$char2hex_executable @original_char_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_char_files; ++$index)
    {
        open(ORIGINAL, '<', $original_hex_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_char2hex_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_hex_files[$index] to $output_char2hex_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_char2hex_files;
    
    print "\n";
}

print "\n";

# bin2char tests
for (my $index = 0; $index <= $#original_bin_files; ++$index)
{
    my $command = "$bin2char_executable $original_bin_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_char_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_bin2char_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_char_files[$index] to $output_bin2char_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_bin2char_files[$index];
    
    print "\n";
}

# bin2char multiple-file test
{
    my $command = "$bin2char_executable @original_bin_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_bin_files; ++$index)
    {
        open(ORIGINAL, '<', $original_char_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_bin2char_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_char_files[$index] to $output_bin2char_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_bin2char_files;
    
    print "\n";
}

# bin2hex tests
for (my $index = 0; $index <= $#original_bin_files; ++$index)
{
    my $command = "$bin2hex_executable $original_bin_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_hex_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_bin2hex_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_hex_files[$index] to $output_bin2hex_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_bin2hex_files[$index];
    
    print "\n";
}

# bin2hex multiple-file test
{
    my $command = "$bin2hex_executable @original_bin_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_bin_files; ++$index)
    {
        open(ORIGINAL, '<', $original_hex_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_bin2hex_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_hex_files[$index] to $output_bin2hex_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_bin2hex_files;
    
    print "\n";
}

print "\n";

# hex2char tests
for (my $index = 0; $index <= $#original_hex_files; ++$index)
{
    my $command = "$hex2char_executable $original_hex_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_char_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_hex2char_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_char_files[$index] to $output_hex2char_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_hex2char_files[$index];
    
    print "\n";
}

# hex2char multiple-file test
{
    my $command = "$hex2char_executable @original_hex_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_hex_files; ++$index)
    {
        open(ORIGINAL, '<', $original_char_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_hex2char_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_char_files[$index] to $output_hex2char_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_hex2char_files;
    
    print "\n";
}

# hex2bin tests
for (my $index = 0; $index <= $#original_hex_files; ++$index)
{
    my $command = "$hex2bin_executable $original_hex_files[$index]";
    print "Executing: $command\n";
    system($command);
    
    open(ORIGINAL, '<', $original_bin_files[$index]);
    my $original_contents = <ORIGINAL>;
    close(ORIGINAL);
    
    open(OUTPUT, '<', $output_hex2bin_files[$index]);
    my $output_contents = <OUTPUT>;
    close(OUTPUT);
    
    print "Comparing $original_bin_files[$index] to $output_hex2bin_files[$index] ... ";
    if ($original_contents eq $output_contents)
    {
        print "PASS\n";
    }
    else
    {
        print "FAIL\n";
    }
    
    print "Removing output files...\n";
    unlink $output_hex2bin_files[$index];
    
    print "\n";
}

# hex2bin multiple-file test
{
    my $command = "$hex2bin_executable @original_hex_files";
    print "Executing: $command\n";
    system($command);
    
    for (my $index = 0; $index <= $#original_hex_files; ++$index)
    {
        open(ORIGINAL, '<', $original_bin_files[$index]);
        my $original_contents = <ORIGINAL>;
        close(ORIGINAL);
        
        open(OUTPUT, '<', $output_hex2bin_files[$index]);
        my $output_contents = <OUTPUT>;
        close(OUTPUT);
        
        print "Comparing $original_bin_files[$index] to $output_hex2bin_files[$index] ... ";
        if ($original_contents eq $output_contents)
        {
            print "PASS\n";
        }
        else
        {
            print "FAIL\n";
        }
        }
    print "Removing output files...\n";
    unlink @output_hex2bin_files;
    
    print "\n";
}

print "\n";

